@@ -3,7 +3,7 @@ use std::cmp::Ordering;
 #[cfg(not(feature = "library"))]
 use cosmwasm_std::entry_point;
 use cosmwasm_std::{
-    attr, to_binary, Binary, BlockInfo, CosmosMsg, Deps, DepsMut, Empty, Env, MessageInfo, Order,
+    to_binary, Binary, BlockInfo, CosmosMsg, Deps, DepsMut, Empty, Env, MessageInfo, Order,
     Response, StdResult,
 };
 
@@ -127,17 +127,11 @@ pub fn execute_propose(
     };
     BALLOTS.save(deps.storage, (id.into(), &info.sender), &ballot)?;
 
-    Ok(Response {
-        submessages: vec![],
-        messages: vec![],
-        attributes: vec![
-            attr("action", "propose"),
-            attr("sender", info.sender),
-            attr("proposal_id", id),
-            attr("status", format!("{:?}", prop.status)),
-        ],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "propose")
+        .add_attribute("sender", info.sender)
+        .add_attribute("proposal_id", id.to_string())
+        .add_attribute("status", format!("{:?}", prop.status)))
 }
 
 pub fn execute_vote(
@@ -183,17 +177,11 @@ pub fn execute_vote(
     prop.update_status(&env.block);
     PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
 
-    Ok(Response {
-        submessages: vec![],
-        messages: vec![],
-        attributes: vec![
-            attr("action", "vote"),
-            attr("sender", info.sender),
-            attr("proposal_id", proposal_id),
-            attr("status", format!("{:?}", prop.status)),
-        ],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "vote")
+        .add_attribute("sender", info.sender)
+        .add_attribute("proposal_id", proposal_id.to_string())
+        .add_attribute("status", format!("{:?}", prop.status)))
 }
 
 pub fn execute_execute(
@@ -216,16 +204,11 @@ pub fn execute_execute(
     PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
 
     // dispatch all proposed messages
-    Ok(Response {
-        submessages: vec![],
-        messages: prop.msgs,
-        attributes: vec![
-            attr("action", "execute"),
-            attr("sender", info.sender),
-            attr("proposal_id", proposal_id),
-        ],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_messages(prop.msgs)
+        .add_attribute("action", "execute")
+        .add_attribute("sender", info.sender)
+        .add_attribute("proposal_id", proposal_id.to_string()))
 }
 
 pub fn execute_close(
@@ -251,16 +234,10 @@ pub fn execute_close(
     prop.status = Status::Rejected;
     PROPOSALS.save(deps.storage, proposal_id.into(), &prop)?;
 
-    Ok(Response {
-        submessages: vec![],
-        messages: vec![],
-        attributes: vec![
-            attr("action", "close"),
-            attr("sender", info.sender),
-            attr("proposal_id", proposal_id),
-        ],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "close")
+        .add_attribute("sender", info.sender)
+        .add_attribute("proposal_id", proposal_id.to_string()))
 }
 
 pub fn execute_membership_hook(
@@ -454,7 +431,7 @@ mod tests {
     use cw2::{query_contract_info, ContractVersion};
     use cw4::{Cw4ExecuteMsg, Member};
     use cw4_group::helpers::Cw4GroupContract;
-    use cw_multi_test::{next_block, App, Contract, ContractWrapper, SimpleBank};
+    use cw_multi_test::{next_block, App, BankKeeper, Contract, ContractWrapper, Executor};
 
     use super::*;
     use crate::msg::Threshold;
@@ -494,10 +471,10 @@ mod tests {
 
     fn mock_app() -> App {
         let env = mock_env();
-        let api = Box::new(MockApi::default());
-        let bank = SimpleBank {};
+        let api = MockApi::default();
+        let bank = BankKeeper::new();
 
-        App::new(api, env.block, bank, || Box::new(MockStorage::new()))
+        App::new(api, env.block, bank, MockStorage::new())
     }
 
     // uploads code and returns address of group contract
@@ -507,7 +484,7 @@ mod tests {
             admin: Some(OWNER.into()),
             members,
         };
-        app.instantiate_contract(group_id, Addr::unchecked(OWNER), &msg, &[], "group")
+        app.instantiate_contract(group_id, Addr::unchecked(OWNER), &msg, &[], "group", None)
             .unwrap()
     }
 
@@ -523,7 +500,7 @@ mod tests {
             threshold,
             max_voting_period,
         };
-        app.instantiate_contract(flex_id, Addr::unchecked(OWNER), &msg, &[], "flex")
+        app.instantiate_contract(flex_id, Addr::unchecked(OWNER), &msg, &[], "flex", None)
             .unwrap()
     }
 
@@ -588,7 +565,7 @@ mod tests {
 
         // Bonus: set some funds on the multisig contract for future proposals
         if !init_funds.is_empty() {
-            app.set_bank_balance(&flex_addr, init_funds).unwrap();
+            app.init_bank_balance(&flex_addr, init_funds).unwrap();
         }
         (flex_addr, group_addr)
     }
@@ -598,7 +575,7 @@ mod tests {
             to_address: SOMEBODY.into(),
             amount: coins(1, "BTC"),
         };
-        let msgs = vec![CosmosMsg::Bank(bank_msg)];
+        let msgs = vec![bank_msg.into()];
         let title = "Pay somebody".to_string();
         let description = "Do I pay her?".to_string();
         (msgs, title, description)
@@ -637,9 +614,10 @@ mod tests {
                 &instantiate_msg,
                 &[],
                 "zero required weight",
+                None,
             )
             .unwrap_err();
-        assert_eq!(err, ContractError::ZeroThreshold {}.to_string());
+        assert_eq!(ContractError::ZeroThreshold {}, err.downcast().unwrap());
 
         // Total weight less than required weight not allowed
         let instantiate_msg = InstantiateMsg {
@@ -654,9 +632,13 @@ mod tests {
                 &instantiate_msg,
                 &[],
                 "high required weight",
+                None,
             )
             .unwrap_err();
-        assert_eq!(err, ContractError::UnreachableThreshold {}.to_string());
+        assert_eq!(
+            ContractError::UnreachableThreshold {},
+            err.downcast().unwrap()
+        );
 
         // All valid
         let instantiate_msg = InstantiateMsg {
@@ -671,6 +653,7 @@ mod tests {
                 &instantiate_msg,
                 &[],
                 "all good",
+                None,
             )
             .unwrap();
 
@@ -723,7 +706,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(SOMEBODY), flex_addr.clone(), &proposal, &[])
             .unwrap_err();
-        assert_eq!(err, ContractError::Unauthorized {}.to_string());
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
 
         // Wrong expiration option fails
         let msgs = match proposal.clone() {
@@ -744,19 +727,19 @@ mod tests {
                 &[],
             )
             .unwrap_err();
-        assert_eq!(err, ContractError::WrongExpiration {}.to_string());
+        assert_eq!(ContractError::WrongExpiration {}, err.downcast().unwrap());
 
         // Proposal from voter works
         let res = app
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "propose"),
-                attr("sender", VOTER3),
-                attr("proposal_id", 1),
-                attr("status", "Open"),
+            res.custom_attrs(1),
+            [
+                ("action", "propose"),
+                ("sender", VOTER3),
+                ("proposal_id", "1"),
+                ("status", "Open"),
             ],
         );
 
@@ -765,12 +748,12 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER4), flex_addr, &proposal, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "propose"),
-                attr("sender", VOTER4),
-                attr("proposal_id", 2),
-                attr("status", "Passed"),
+            res.custom_attrs(1),
+            [
+                ("action", "propose"),
+                ("sender", VOTER4),
+                ("proposal_id", "2"),
+                ("status", "Passed"),
             ],
         );
     }
@@ -832,7 +815,7 @@ mod tests {
         let res = app
             .execute_contract(Addr::unchecked(VOTER1), flex_addr.clone(), &proposal, &[])
             .unwrap();
-        let proposal_id1: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id1: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // another proposal immediately passes
         app.update_block(next_block);
@@ -840,7 +823,7 @@ mod tests {
         let res = app
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
             .unwrap();
-        let proposal_id2: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id2: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // expire them both
         app.update_block(expire(voting_period));
@@ -850,7 +833,7 @@ mod tests {
         let res = app
             .execute_contract(Addr::unchecked(VOTER2), flex_addr.clone(), &proposal, &[])
             .unwrap();
-        let proposal_id3: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id3: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         let proposed_at = app.block_info();
 
         // next block, let's query them all... make sure status is properly updated (1 should be rejected in query)
@@ -930,7 +913,7 @@ mod tests {
             .unwrap();
 
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // Owner cannot vote (again)
         let yes_vote = ExecuteMsg::Vote {
@@ -940,25 +923,25 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::AlreadyVoted {}.to_string(), err);
+        assert_eq!(ContractError::AlreadyVoted {}, err.downcast().unwrap());
 
         // Only voters can vote
         let err = app
             .execute_contract(Addr::unchecked(SOMEBODY), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::Unauthorized {}.to_string(), err);
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
 
         // But voter1 can
         let res = app
             .execute_contract(Addr::unchecked(VOTER1), flex_addr.clone(), &yes_vote, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "vote"),
-                attr("sender", VOTER1),
-                attr("proposal_id", proposal_id),
-                attr("status", "Open"),
+            res.custom_attrs(1),
+            [
+                ("action", "vote"),
+                ("sender", VOTER1),
+                ("proposal_id", proposal_id.to_string().as_str()),
+                ("status", "Open"),
             ],
         );
 
@@ -991,14 +974,14 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::AlreadyVoted {}.to_string(), err);
+        assert_eq!(ContractError::AlreadyVoted {}, err.downcast().unwrap());
 
         // Expired proposals cannot be voted
         app.update_block(expire(voting_period));
         let err = app
             .execute_contract(Addr::unchecked(VOTER4), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::Expired {}.to_string(), err);
+        assert_eq!(ContractError::Expired {}, err.downcast().unwrap());
         app.update_block(unexpire(voting_period));
 
         // Powerful voter supports it, so it passes
@@ -1006,12 +989,12 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER4), flex_addr.clone(), &yes_vote, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "vote"),
-                attr("sender", VOTER4),
-                attr("proposal_id", proposal_id),
-                attr("status", "Passed"),
+            res.custom_attrs(1),
+            [
+                ("action", "vote"),
+                ("sender", VOTER4),
+                ("proposal_id", proposal_id.to_string().as_str()),
+                ("status", "Passed"),
             ],
         );
 
@@ -1019,7 +1002,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(VOTER5), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::NotOpen {}.to_string(), err);
+        assert_eq!(ContractError::NotOpen {}, err.downcast().unwrap());
 
         // query individual votes
         // initial (with 0 weight)
@@ -1086,14 +1069,17 @@ mod tests {
             .unwrap();
 
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // Only Passed can be executed
         let execution = ExecuteMsg::Execute { proposal_id };
         let err = app
             .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &execution, &[])
             .unwrap_err();
-        assert_eq!(ContractError::WrongExecuteStatus {}.to_string(), err);
+        assert_eq!(
+            ContractError::WrongExecuteStatus {},
+            err.downcast().unwrap()
+        );
 
         // Vote it, so it passes
         let vote = ExecuteMsg::Vote {
@@ -1104,12 +1090,12 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &vote, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "vote"),
-                attr("sender", VOTER3),
-                attr("proposal_id", proposal_id),
-                attr("status", "Passed"),
+            res.custom_attrs(1),
+            [
+                ("action", "vote"),
+                ("sender", VOTER3),
+                ("proposal_id", proposal_id.to_string().as_str()),
+                ("status", "Passed"),
             ],
         );
 
@@ -1118,7 +1104,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(OWNER), flex_addr.clone(), &closing, &[])
             .unwrap_err();
-        assert_eq!(ContractError::WrongCloseStatus {}.to_string(), err);
+        assert_eq!(ContractError::WrongCloseStatus {}, err.downcast().unwrap());
 
         // Execute works. Anybody can execute Passed proposals
         let res = app
@@ -1130,11 +1116,11 @@ mod tests {
             )
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "execute"),
-                attr("sender", SOMEBODY),
-                attr("proposal_id", proposal_id),
+            res.custom_attrs(1),
+            [
+                ("action", "execute"),
+                ("sender", SOMEBODY),
+                ("proposal_id", proposal_id.to_string().as_str()),
             ],
         );
 
@@ -1148,7 +1134,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(OWNER), flex_addr, &closing, &[])
             .unwrap_err();
-        assert_eq!(ContractError::WrongCloseStatus {}.to_string(), err);
+        assert_eq!(ContractError::WrongCloseStatus {}, err.downcast().unwrap());
     }
 
     #[test]
@@ -1172,14 +1158,14 @@ mod tests {
             .unwrap();
 
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // Non-expired proposals cannot be closed
         let closing = ExecuteMsg::Close { proposal_id };
         let err = app
             .execute_contract(Addr::unchecked(SOMEBODY), flex_addr.clone(), &closing, &[])
             .unwrap_err();
-        assert_eq!(ContractError::NotExpired {}.to_string(), err);
+        assert_eq!(ContractError::NotExpired {}, err.downcast().unwrap());
 
         // Expired proposals can be closed
         app.update_block(expire(voting_period));
@@ -1187,11 +1173,11 @@ mod tests {
             .execute_contract(Addr::unchecked(SOMEBODY), flex_addr.clone(), &closing, &[])
             .unwrap();
         assert_eq!(
-            res.attributes,
-            vec![
-                attr("action", "close"),
-                attr("sender", SOMEBODY),
-                attr("proposal_id", proposal_id),
+            res.custom_attrs(1),
+            [
+                ("action", "close"),
+                ("sender", SOMEBODY),
+                ("proposal_id", proposal_id.to_string().as_str()),
             ],
         );
 
@@ -1200,7 +1186,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(SOMEBODY), flex_addr, &closing, &[])
             .unwrap_err();
-        assert_eq!(ContractError::WrongCloseStatus {}.to_string(), err);
+        assert_eq!(ContractError::WrongCloseStatus {}, err.downcast().unwrap());
     }
 
     // uses the power from the beginning of the voting period
@@ -1224,7 +1210,7 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER1), flex_addr.clone(), &proposal, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         let prop_status = |app: &App, proposal_id: u64| -> Status {
             let query_prop = QueryMsg::Proposal { proposal_id };
             let prop: ProposalResponse = app
@@ -1285,7 +1271,7 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER1), flex_addr.clone(), &proposal2, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id2: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id2: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // VOTER2 can pass this alone with the updated vote (newer height ignores snapshot)
         let yes_vote = ExecuteMsg::Vote {
@@ -1309,7 +1295,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(newbie), flex_addr.clone(), &yes_vote, &[])
             .unwrap_err();
-        assert_eq!(ContractError::Unauthorized {}.to_string(), err);
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
 
         // previously removed VOTER3 can still vote, passing the proposal
         app.execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &yes_vote, &[])
@@ -1366,7 +1352,7 @@ mod tests {
             )
             .unwrap();
         // Get the proposal id from the logs
-        let update_proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let update_proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // next block...
         app.update_block(|b| b.height += 1);
@@ -1382,7 +1368,7 @@ mod tests {
             )
             .unwrap();
         // Get the proposal id from the logs
-        let cash_proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let cash_proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         assert_ne!(cash_proposal_id, update_proposal_id);
 
         // query proposal state
@@ -1440,7 +1426,7 @@ mod tests {
                 &[],
             )
             .unwrap_err();
-        assert_eq!(ContractError::Unauthorized {}.to_string(), err);
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
 
         // extra: ensure no one else can call the hook
         let hook_hack = ExecuteMsg::MemberChangedHook(MemberChangedHookMsg {
@@ -1449,7 +1435,7 @@ mod tests {
         let err = app
             .execute_contract(Addr::unchecked(VOTER2), flex_addr.clone(), &hook_hack, &[])
             .unwrap_err();
-        assert_eq!(ContractError::Unauthorized {}.to_string(), err);
+        assert_eq!(ContractError::Unauthorized {}, err.downcast().unwrap());
     }
 
     // uses the power from the beginning of the voting period
@@ -1475,7 +1461,7 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         let prop_status = |app: &App| -> Status {
             let query_prop = QueryMsg::Proposal { proposal_id };
             let prop: ProposalResponse = app
@@ -1519,7 +1505,7 @@ mod tests {
             .execute_contract(Addr::unchecked(newbie), flex_addr.clone(), &proposal, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id2: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id2: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
 
         // check proposal2 status
         let query_prop = QueryMsg::Proposal {
@@ -1557,7 +1543,7 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER3), flex_addr.clone(), &proposal, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         let prop_status = |app: &App| -> Status {
             let query_prop = QueryMsg::Proposal { proposal_id };
             let prop: ProposalResponse = app
@@ -1626,7 +1612,7 @@ mod tests {
             .execute_contract(Addr::unchecked(VOTER5), flex_addr.clone(), &proposal, &[])
             .unwrap();
         // Get the proposal id from the logs
-        let proposal_id: u64 = res.attributes[2].value.parse().unwrap();
+        let proposal_id: u64 = res.custom_attrs(1)[2].value.parse().unwrap();
         let prop_status = |app: &App| -> Status {
             let query_prop = QueryMsg::Proposal { proposal_id };
             let prop: ProposalResponse = app
